@@ -5,19 +5,19 @@ package com.googlecode.mp4parser.authoring;
  */
 public class Edit {
     private long timeScale;
-    private double segmentDurationInMs;
+    private double segmentDuration;
     private long mediaTime;
     private double mediaRate;
 
     /**
      * @param mediaTime           time within the current track that is considered start time of this edit.
      * @param timeScale           time scale of the media time entry
-     * @param segmentDurationInMs segment duration in milliseconds
+     * @param segmentDurationInMs segment duration in seconds
      * @param mediaRate           when mediaRate is 1.0 the playback will be normal. When 2.0 it will be twice as fast.
      */
     public Edit(long mediaTime, long timeScale, double mediaRate, double segmentDurationInMs) {
         this.timeScale = timeScale;
-        this.segmentDurationInMs = segmentDurationInMs;
+        this.segmentDuration = segmentDurationInMs;
         this.mediaTime = mediaTime;
         this.mediaRate = mediaRate;
     }
@@ -27,7 +27,7 @@ public class Edit {
     }
 
     public double getSegmentDuration() {
-        return segmentDurationInMs;
+        return segmentDuration;
     }
 
     public long getMediaTime() {
