@@ -25,11 +25,13 @@ import com.googlecode.mp4parser.authoring.Edit;
 import com.googlecode.mp4parser.authoring.Movie;
 import com.googlecode.mp4parser.authoring.Sample;
 import com.googlecode.mp4parser.authoring.Track;
-import com.googlecode.mp4parser.authoring.tracks.CencEncyprtedTrack;
+import com.googlecode.mp4parser.authoring.tracks.CencEncryptedTrack;
 import com.googlecode.mp4parser.boxes.dece.SampleEncryptionBox;
 import com.googlecode.mp4parser.boxes.mp4.samplegrouping.GroupEntry;
 import com.googlecode.mp4parser.boxes.mp4.samplegrouping.SampleGroupDescriptionBox;
 import com.googlecode.mp4parser.boxes.mp4.samplegrouping.SampleToGroupBox;
+import com.googlecode.mp4parser.util.Logger;
+import com.googlecode.mp4parser.util.Mp4Arrays;
 import com.googlecode.mp4parser.util.Path;
 import com.mp4parser.iso14496.part12.SampleAuxiliaryInformationOffsetsBox;
 import com.mp4parser.iso14496.part12.SampleAuxiliaryInformationSizesBox;
@@ -39,22 +41,21 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.WritableByteChannel;
 import java.util.*;
-import java.util.logging.Level;
-import java.util.logging.Logger;
 
 import static com.googlecode.mp4parser.util.CastUtils.l2i;
+import static com.googlecode.mp4parser.util.Math.lcm;
 
 /**
  * Creates a plain MP4 file from a video. Plain as plain can be.
  */
 public class DefaultMp4Builder implements Mp4Builder {
 
-    private static Logger LOG = Logger.getLogger(DefaultMp4Builder.class.getName());
-    Set<StaticChunkOffsetBox> chunkOffsetBoxes = new HashSet<StaticChunkOffsetBox>();
+    private static Logger LOG = Logger.getLogger(DefaultMp4Builder.class);
+    Map<Track, StaticChunkOffsetBox> chunkOffsetBoxes = new HashMap<Track, StaticChunkOffsetBox>();
     Set<SampleAuxiliaryInformationOffsetsBox> sampleAuxiliaryInformationOffsetsBoxes = new HashSet<SampleAuxiliaryInformationOffsetsBox>();
     HashMap<Track, List<Sample>> track2Sample = new HashMap<Track, List<Sample>>();
     HashMap<Track, long[]> track2SampleSizes = new HashMap<Track, long[]>();
-    private FragmentIntersectionFinder intersectionFinder;
+    private Fragmenter fragmenter;
 
     private static long sum(int[] ls) {
         long rc = 0;
@@ -79,18 +80,18 @@ public class DefaultMp4Builder implements Mp4Builder {
         return gcd(b, a % b);
     }
 
-    public void setIntersectionFinder(FragmentIntersectionFinder intersectionFinder) {
-        this.intersectionFinder = intersectionFinder;
+    public void setFragmenter(Fragmenter fragmenter) {
+        this.fragmenter = fragmenter;
     }
 
     /**
      * {@inheritDoc}
      */
     public Container build(Movie movie) {
-        if (intersectionFinder == null) {
-            intersectionFinder = new TwoSecondIntersectionFinder(movie, 2);
+        if (fragmenter == null) {
+            fragmenter = new BetterFragmenter(2);
         }
-        LOG.fine("Creating movie " + movie);
+        LOG.logDebug("Creating movie " + movie);
         for (Track track : movie.getTracks()) {
             // getting the samples may be a time consuming activity
             List<Sample> samples = track.getSamples();
@@ -110,7 +111,7 @@ public class DefaultMp4Builder implements Mp4Builder {
 
         Map<Track, int[]> chunks = new HashMap<Track, int[]>();
         for (Track track : movie.getTracks()) {
-            chunks.put(track, getChunkSizes(track, movie));
+            chunks.put(track, getChunkSizes(track));
         }
         Box moov = createMovieBox(movie, chunks);
         isoFile.addBox(moov);
@@ -121,16 +122,17 @@ public class DefaultMp4Builder implements Mp4Builder {
             contentSize += sum(stsz.getSampleSizes());
 
         }
-
+        LOG.logDebug("About to create mdat");
         InterleaveChunkMdat mdat = new InterleaveChunkMdat(movie, chunks, contentSize);
         isoFile.addBox(mdat);
+        LOG.logDebug("mdat crated");
 
         /*
         dataOffset is where the first sample starts. In this special mdat the samples always start
         at offset 16 so that we can use the same offset for large boxes and small boxes
          */
         long dataOffset = mdat.getDataOffset();
-        for (StaticChunkOffsetBox chunkOffsetBox : chunkOffsetBoxes) {
+        for (StaticChunkOffsetBox chunkOffsetBox : chunkOffsetBoxes.values()) {
             long[] offsets = chunkOffsetBox.getChunkOffsets();
             for (int i = 0; i < offsets.length; i++) {
                 offsets[i] += dataOffset;
@@ -173,11 +175,11 @@ public class DefaultMp4Builder implements Mp4Builder {
     protected FileTypeBox createFileTypeBox(Movie movie) {
         List<String> minorBrands = new LinkedList<String>();
 
-        minorBrands.add("isom");
-        minorBrands.add("iso2");
+        minorBrands.add("mp42");
+        minorBrands.add("iso6");
         minorBrands.add("avc1");
-
-        return new FileTypeBox("isom", 0, minorBrands);
+        minorBrands.add("isom");
+        return new FileTypeBox("iso6", 1, minorBrands);
     }
 
     protected MovieBox createMovieBox(Movie movie, Map<Track, int[]> chunks) {
@@ -191,7 +193,19 @@ public class DefaultMp4Builder implements Mp4Builder {
         long duration = 0;
 
         for (Track track : movie.getTracks()) {
-            long tracksDuration = track.getDuration() * movieTimeScale / track.getTrackMetaData().getTimescale();
+            long tracksDuration;
+
+            if (track.getEdits() == null || track.getEdits().isEmpty()) {
+                tracksDuration = (track.getDuration() * movieTimeScale / track.getTrackMetaData().getTimescale());
+            } else {
+                double d = 0;
+                for (Edit edit : track.getEdits()) {
+                    d += (long) edit.getSegmentDuration();
+                }
+                tracksDuration = (long) (d * movieTimeScale);
+            }
+
+
             if (tracksDuration > duration) {
                 duration = tracksDuration;
             }
@@ -238,16 +252,24 @@ public class DefaultMp4Builder implements Mp4Builder {
 
         tkhd.setEnabled(true);
         tkhd.setInMovie(true);
-        tkhd.setInPreview(true);
-        tkhd.setInPoster(true);
+//        tkhd.setInPreview(true);
+//        tkhd.setInPoster(true);
         tkhd.setMatrix(track.getTrackMetaData().getMatrix());
 
         tkhd.setAlternateGroup(track.getTrackMetaData().getGroup());
         tkhd.setCreationTime(track.getTrackMetaData().getCreationTime());
-        // We need to take edit list box into account in trackheader duration
-        // but as long as I don't support edit list boxes it is sufficient to
-        // just translate media duration to movie timescale
-        tkhd.setDuration(track.getDuration() * getTimescale(movie) / track.getTrackMetaData().getTimescale());
+
+        if (track.getEdits() == null || track.getEdits().isEmpty()) {
+            tkhd.setDuration(track.getDuration() * getTimescale(movie) / track.getTrackMetaData().getTimescale());
+        } else {
+            long d = 0;
+            for (Edit edit : track.getEdits()) {
+                d += (long) edit.getSegmentDuration();
+            }
+            tkhd.setDuration(d * track.getTrackMetaData().getTimescale());
+        }
+
+
         tkhd.setHeight(track.getTrackMetaData().getHeight());
         tkhd.setWidth(track.getTrackMetaData().getWidth());
         tkhd.setLayer(track.getTrackMetaData().getLayer());
@@ -301,19 +323,19 @@ public class DefaultMp4Builder implements Mp4Builder {
         Box stbl = createStbl(track, movie, chunks);
         minf.addBox(stbl);
         mdia.addBox(minf);
-
+        LOG.logDebug("done with trak for track_" + track.getTrackMetaData().getTrackId());
         return trackBox;
     }
 
     protected Box createEdts(Track track, Movie movie) {
         if (track.getEdits() != null && track.getEdits().size() > 0) {
             EditListBox elst = new EditListBox();
-            elst.setVersion(1);
+            elst.setVersion(0); // quicktime won't play file when version = 1
             List<EditListBox.Entry> entries = new ArrayList<EditListBox.Entry>();
 
             for (Edit edit : track.getEdits()) {
                 entries.add(new EditListBox.Entry(elst,
-                        (long) edit.getSegmentDuration() * movie.getTimescale(),
+                        Math.round(edit.getSegmentDuration() * movie.getTimescale()),
                         edit.getMediaTime() * track.getTrackMetaData().getTimescale() / edit.getTimeScale(),
                         edit.getMediaRate()));
             }
@@ -339,6 +361,7 @@ public class DefaultMp4Builder implements Mp4Builder {
         createStsz(track, stbl);
         createStco(track, movie, chunks, stbl);
 
+
         Map<String, List<GroupEntry>> groupEntryFamilies = new HashMap<String, List<GroupEntry>>();
         for (Map.Entry<GroupEntry, long[]> sg : track.getSampleGroups().entrySet()) {
             String type = sg.getKey().getType();
@@ -352,6 +375,7 @@ public class DefaultMp4Builder implements Mp4Builder {
         for (Map.Entry<String, List<GroupEntry>> sg : groupEntryFamilies.entrySet()) {
             SampleGroupDescriptionBox sgdb = new SampleGroupDescriptionBox();
             String type = sg.getKey();
+            sgdb.setGroupingType(type);
             sgdb.setGroupEntries(sg.getValue());
             SampleToGroupBox sbgp = new SampleToGroupBox();
             sbgp.setGroupingType(type);
@@ -376,11 +400,11 @@ public class DefaultMp4Builder implements Mp4Builder {
             stbl.addBox(sbgp);
         }
 
-        if (track instanceof CencEncyprtedTrack) {
-            createCencBoxes((CencEncyprtedTrack) track, stbl, chunks.get(track));
+        if (track instanceof CencEncryptedTrack) {
+            createCencBoxes((CencEncryptedTrack) track, stbl, chunks.get(track));
         }
         createSubs(track, stbl);
-
+        LOG.logDebug("done with stbl for track_" + track.getTrackMetaData().getTrackId());
         return stbl;
     }
 
@@ -390,7 +414,7 @@ public class DefaultMp4Builder implements Mp4Builder {
         }
     }
 
-    protected void createCencBoxes(CencEncyprtedTrack track, SampleTableBox stbl, int[] chunkSizes) {
+    protected void createCencBoxes(CencEncryptedTrack track, SampleTableBox stbl, int[] chunkSizes) {
 
         SampleAuxiliaryInformationSizesBox saiz = new SampleAuxiliaryInformationSizesBox();
 
@@ -438,49 +462,70 @@ public class DefaultMp4Builder implements Mp4Builder {
         stbl.addBox(track.getSampleDescriptionBox());
     }
 
-    protected void createStco(Track track, Movie movie, Map<Track, int[]> chunks, SampleTableBox stbl) {
-        int[] tracksChunkSizes = chunks.get(track);
+    protected void createStco(Track targetTrack, Movie movie, Map<Track, int[]> chunks, SampleTableBox stbl) {
+        if (chunkOffsetBoxes.get(targetTrack) == null) {
+            // The ChunkOffsetBox we create here is just a stub
+            // since we haven't created the whole structure we can't tell where the
+            // first chunk starts (mdat box). So I just let the chunk offset
+            // start at zero and I will add the mdat offset later.
+
+            long offset = 0;
+            // all tracks have the same number of chunks
+            LOG.logDebug("Calculating chunk offsets for track_" + targetTrack.getTrackMetaData().getTrackId());
+
+            List<Track> tracks = new ArrayList<Track>(chunks.keySet());
+            Collections.sort(tracks, new Comparator<Track>() {
+                public int compare(Track o1, Track o2) {
+                    return l2i(o1.getTrackMetaData().getTrackId() - o2.getTrackMetaData().getTrackId());
+                }
+            });
+            Map<Track, Integer> trackToChunk = new HashMap<Track, Integer>();
+            Map<Track, Integer> trackToSample = new HashMap<Track, Integer>();
+            Map<Track, Double> trackToTime = new HashMap<Track, Double>();
+            for (Track track : tracks) {
+                trackToChunk.put(track, 0);
+                trackToSample.put(track, 0);
+                trackToTime.put(track, 0.0);
+                chunkOffsetBoxes.put(track, new StaticChunkOffsetBox());
+            }
+
+            while (true) {
+                Track nextChunksTrack = null;
+                for (Track track : tracks) {
+                    // This always chooses the least progressed track
+                    if ((nextChunksTrack == null || trackToTime.get(track) < trackToTime.get(nextChunksTrack)) &&
+                            // either first OR track's next chunk's starttime is smaller than nextTrack's next chunks starttime
+                            // AND their need to be chunks left!
+                            (trackToChunk.get(track) < chunks.get(track).length)) {
+                        nextChunksTrack = track;
+                    }
+                }
+                if (nextChunksTrack == null) {
+                    break; // no next
+                }
+                // found the next one
+                ChunkOffsetBox chunkOffsetBox = chunkOffsetBoxes.get(nextChunksTrack);
+                chunkOffsetBox.setChunkOffsets(Mp4Arrays.copyOfAndAppend(chunkOffsetBox.getChunkOffsets(), offset));
+
+                int nextChunksIndex = trackToChunk.get(nextChunksTrack);
+
+                int numberOfSampleInNextChunk = chunks.get(nextChunksTrack)[nextChunksIndex];
+                int startSample = trackToSample.get(nextChunksTrack);
+                double time = trackToTime.get(nextChunksTrack);
+
+                long[] durs = nextChunksTrack.getSampleDurations();
+                for (int j = startSample; j < startSample + numberOfSampleInNextChunk; j++) {
+                    offset += track2SampleSizes.get(nextChunksTrack)[j];
+                    time += (double) durs[j] / nextChunksTrack.getTrackMetaData().getTimescale();
+                }
+                trackToChunk.put(nextChunksTrack, nextChunksIndex + 1);
+                trackToSample.put(nextChunksTrack, startSample + numberOfSampleInNextChunk);
+                trackToTime.put(nextChunksTrack, time);
+            }
 
-        // The ChunkOffsetBox we create here is just a stub
-        // since we haven't created the whole structure we can't tell where the
-        // first chunk starts (mdat box). So I just let the chunk offset
-        // start at zero and I will add the mdat offset later.
-        StaticChunkOffsetBox stco = new StaticChunkOffsetBox();
-        this.chunkOffsetBoxes.add(stco);
-        long offset = 0;
-        long[] chunkOffset = new long[tracksChunkSizes.length];
-        // all tracks have the same number of chunks
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.fine("Calculating chunk offsets for track_" + track.getTrackMetaData().getTrackId());
         }
 
-
-        for (int i = 0; i < tracksChunkSizes.length; i++) {
-            // The filelayout will be:
-            // chunk_1_track_1,... ,chunk_1_track_n, chunk_2_track_1,... ,chunk_2_track_n, ... , chunk_m_track_1,... ,chunk_m_track_n
-            // calculating the offsets
-            if (LOG.isLoggable(Level.FINER)) {
-                LOG.finer("Calculating chunk offsets for track_" + track.getTrackMetaData().getTrackId() + " chunk " + i);
-            }
-            for (Track current : movie.getTracks()) {
-                if (LOG.isLoggable(Level.FINEST)) {
-                    LOG.finest("Adding offsets of track_" + current.getTrackMetaData().getTrackId());
-                }
-                int[] chunkSizes = chunks.get(current);
-                long firstSampleOfChunk = 0;
-                for (int j = 0; j < i; j++) {
-                    firstSampleOfChunk += chunkSizes[j];
-                }
-                if (current == track) {
-                    chunkOffset[i] = offset;
-                }
-                for (int j = l2i(firstSampleOfChunk); j < firstSampleOfChunk + chunkSizes[i]; j++) {
-                    offset += track2SampleSizes.get(current)[j];
-                }
-            }
-        }
-        stco.setChunkOffsets(chunkOffset);
-        stbl.addBox(stco);
+        stbl.addBox(chunkOffsetBoxes.get(targetTrack));
     }
 
     protected void createStsz(Track track, SampleTableBox stbl) {
@@ -556,13 +601,12 @@ public class DefaultMp4Builder implements Mp4Builder {
     /**
      * Gets the chunk sizes for the given track.
      *
-     * @param track
-     * @param movie
-     * @return
+     * @param track the track we are talking about
+     * @return the size of each chunk in number of samples
      */
-    int[] getChunkSizes(Track track, Movie movie) {
+    int[] getChunkSizes(Track track) {
 
-        long[] referenceChunkStarts = intersectionFinder.sampleNumbers(track);
+        long[] referenceChunkStarts = fragmenter.sampleNumbers(track);
         int[] chunkSizes = new int[referenceChunkStarts.length];
 
 
@@ -576,7 +620,6 @@ public class DefaultMp4Builder implements Mp4Builder {
             }
 
             chunkSizes[i] = l2i(end - start);
-            // The Stretch makes sure that there are as much audio and video chunks!
         }
         assert DefaultMp4Builder.this.track2Sample.get(track).size() == sum(chunkSizes) : "The number of samples and the sum of all chunk lengths must be equal";
         return chunkSizes;
@@ -585,9 +628,10 @@ public class DefaultMp4Builder implements Mp4Builder {
     }
 
     public long getTimescale(Movie movie) {
+
         long timescale = movie.getTracks().iterator().next().getTrackMetaData().getTimescale();
         for (Track track : movie.getTracks()) {
-            timescale = gcd(track.getTrackMetaData().getTimescale(), timescale);
+            timescale = lcm(timescale, track.getTrackMetaData().getTimescale());
         }
         return timescale;
     }
@@ -602,21 +646,51 @@ public class DefaultMp4Builder implements Mp4Builder {
         private InterleaveChunkMdat(Movie movie, Map<Track, int[]> chunks, long contentSize) {
             this.contentSize = contentSize;
             this.tracks = movie.getTracks();
-
-            for (int i = 0; i < chunks.values().iterator().next().length; i++) {
-                for (Track track : tracks) {
-
-                    int[] chunkSizes = chunks.get(track);
-                    long firstSampleOfChunk = 0;
-                    for (int j = 0; j < i; j++) {
-                        firstSampleOfChunk += chunkSizes[j];
-                    }
-                    List<Sample> chunk = DefaultMp4Builder.this.track2Sample.get(track).subList(l2i(firstSampleOfChunk), l2i(firstSampleOfChunk + chunkSizes[i]));
-                    chunkList.add(chunk);
+            List<Track> tracks = new ArrayList<Track>(chunks.keySet());
+            Collections.sort(tracks, new Comparator<Track>() {
+                public int compare(Track o1, Track o2) {
+                    return l2i(o1.getTrackMetaData().getTrackId() - o2.getTrackMetaData().getTrackId());
                 }
-
+            });
+            Map<Track, Integer> trackToChunk = new HashMap<Track, Integer>();
+            Map<Track, Integer> trackToSample = new HashMap<Track, Integer>();
+            Map<Track, Double> trackToTime = new HashMap<Track, Double>();
+            for (Track track : tracks) {
+                trackToChunk.put(track, 0);
+                trackToSample.put(track, 0);
+                trackToTime.put(track, 0.0);
             }
 
+            while (true) {
+                Track nextChunksTrack = null;
+                for (Track track : tracks) {
+                    if ((nextChunksTrack == null || trackToTime.get(track) < trackToTime.get(nextChunksTrack)) &&
+                            // either first OR track's next chunk's starttime is smaller than nextTrack's next chunks starttime
+                            // AND their need to be chunks left!
+                            (trackToChunk.get(track) < chunks.get(track).length)) {
+                        nextChunksTrack = track;
+                    }
+                }
+                if (nextChunksTrack == null) {
+                    break;
+                }
+                // found the next one
+
+                int nextChunksIndex = trackToChunk.get(nextChunksTrack);
+                int numberOfSampleInNextChunk = chunks.get(nextChunksTrack)[nextChunksIndex];
+                int startSample = trackToSample.get(nextChunksTrack);
+                double time = trackToTime.get(nextChunksTrack);
+                for (int j = startSample; j < startSample + numberOfSampleInNextChunk; j++) {
+                    time += (double) nextChunksTrack.getSampleDurations()[j] / nextChunksTrack.getTrackMetaData().getTimescale();
+                }
+                chunkList.add(nextChunksTrack.getSamples().subList(startSample, startSample + numberOfSampleInNextChunk));
+
+                trackToChunk.put(nextChunksTrack, nextChunksIndex + 1);
+                trackToSample.put(nextChunksTrack, startSample + numberOfSampleInNextChunk);
+                trackToTime.put(nextChunksTrack, time);
+            }
+
+
         }
 
         public Container getParent() {
@@ -679,9 +753,19 @@ public class DefaultMp4Builder implements Mp4Builder {
             }
             bb.rewind();
             writableByteChannel.write(bb);
+            long writtenBytes = 0;
+            long writtenMegaBytes = 0;
+
+            LOG.logDebug("About to write " + contentSize);
             for (List<Sample> samples : chunkList) {
                 for (Sample sample : samples) {
                     sample.writeTo(writableByteChannel);
+                    writtenBytes += sample.getSize();
+                    if (writtenBytes > 1024 * 1024) {
+                        writtenBytes -= 1024 * 1024;
+                        writtenMegaBytes++;
+                        LOG.logDebug("Written " + writtenMegaBytes + "MB");
+                    }
                 }
             }
 
