@@ -26,7 +26,7 @@ import com.googlecode.mp4parser.authoring.Edit;
 import com.googlecode.mp4parser.authoring.Movie;
 import com.googlecode.mp4parser.authoring.Sample;
 import com.googlecode.mp4parser.authoring.Track;
-import com.googlecode.mp4parser.authoring.tracks.CencEncyprtedTrack;
+import com.googlecode.mp4parser.authoring.tracks.CencEncryptedTrack;
 import com.googlecode.mp4parser.boxes.dece.SampleEncryptionBox;
 import com.googlecode.mp4parser.boxes.mp4.samplegrouping.GroupEntry;
 import com.googlecode.mp4parser.boxes.mp4.samplegrouping.SampleGroupDescriptionBox;
@@ -51,7 +51,7 @@ import static com.googlecode.mp4parser.util.CastUtils.l2i;
 public class FragmentedMp4Builder implements Mp4Builder {
     private static final Logger LOG = Logger.getLogger(FragmentedMp4Builder.class.getName());
 
-    protected FragmentIntersectionFinder intersectionFinder;
+    protected Fragmenter fragmenter;
 
     public FragmentedMp4Builder() {
     }
@@ -62,10 +62,11 @@ public class FragmentedMp4Builder implements Mp4Builder {
 
     public Box createFtyp(Movie movie) {
         List<String> minorBrands = new LinkedList<String>();
-        minorBrands.add("isom");
-        minorBrands.add("iso2");
+        minorBrands.add("mp42");
+        minorBrands.add("iso6");
         minorBrands.add("avc1");
-        return new FileTypeBox("isom", 0, minorBrands);
+        minorBrands.add("isom");
+        return new FileTypeBox("iso6", 1, minorBrands);
     }
 
     /**
@@ -109,15 +110,55 @@ public class FragmentedMp4Builder implements Mp4Builder {
     protected List<Box> createMoofMdat(final Movie movie) {
         List<Box> moofsMdats = new LinkedList<Box>();
         HashMap<Track, long[]> intersectionMap = new HashMap<Track, long[]>();
-        int maxNumberOfFragments = 0;
+        HashMap<Track, Double> track2currentTime = new HashMap<Track, Double>();
+
         for (Track track : movie.getTracks()) {
-            long[] intersects = intersectionFinder.sampleNumbers(track);
+            long[] intersects = fragmenter.sampleNumbers(track);
             intersectionMap.put(track, intersects);
-            maxNumberOfFragments = Math.max(maxNumberOfFragments, intersects.length);
+            track2currentTime.put(track, 0.0);
+
         }
 
-
         int sequence = 1;
+        while (!intersectionMap.isEmpty()) {
+            Track earliestTrack = null;
+            double earliestTime = Double.MAX_VALUE;
+            for (Map.Entry<Track, Double> trackEntry : track2currentTime.entrySet()) {
+                if (trackEntry.getValue() < earliestTime) {
+                    earliestTime = trackEntry.getValue();
+                    earliestTrack = trackEntry.getKey();
+                }
+            }
+            assert earliestTrack != null;
+
+            long[] startSamples = intersectionMap.get(earliestTrack);
+            long startSample = startSamples[0];
+            long endSample = startSamples.length > 1 ? startSamples[1] : earliestTrack.getSamples().size() + 1;
+
+            long[] times = earliestTrack.getSampleDurations();
+            long timscale = earliestTrack.getTrackMetaData().getTimescale();
+            for (long i = startSample; i < endSample; i++) {
+                earliestTime += (double) times[l2i(i-1)] / timscale;
+            }
+            createFragment(moofsMdats, earliestTrack, startSample, endSample, sequence);
+
+            if (startSamples.length == 1) {
+                intersectionMap.remove(earliestTrack);
+                track2currentTime.remove(earliestTrack);
+                // all sample written.
+            } else {
+                long[] nuStartSamples = new long[startSamples.length - 1];
+                System.arraycopy(startSamples, 1, nuStartSamples, 0, nuStartSamples.length);
+                intersectionMap.put(earliestTrack, nuStartSamples);
+                track2currentTime.put(earliestTrack, earliestTime);
+            }
+
+
+            sequence++;
+
+        }
+
+      /* sequence = 1;
         // this loop has two indices:
 
         for (int cycle = 0; cycle < maxNumberOfFragments; cycle++) {
@@ -126,25 +167,23 @@ public class FragmentedMp4Builder implements Mp4Builder {
 
             for (Track track : sortedTracks) {
                 long[] startSamples = intersectionMap.get(track);
-                sequence = createFragment(moofsMdats, track, startSamples, cycle, sequence);
+                long startSample = startSamples[cycle];
+                // one based sample numbers - the first sample is 1
+                long endSample = cycle + 1 < startSamples.length ? startSamples[cycle + 1] : track.getSamples().size() + 1;
+                createFragment(moofsMdats, track, startSample, endSample, sequence);
+                sequence++;
             }
-        }
+        }*/
         return moofsMdats;
     }
 
-    protected int createFragment(List<Box> moofsMdats, Track track, long[] startSamples, int cycle, int sequence) {
-        //some tracks may have less fragments -> skip them
-        if (cycle < startSamples.length) {
+    protected int createFragment(List<Box> moofsMdats, Track track, long startSample, long endSample, int sequence) {
 
-            long startSample = startSamples[cycle];
-            // one based sample numbers - the first sample is 1
-            long endSample = cycle + 1 < startSamples.length ? startSamples[cycle + 1] : track.getSamples().size() + 1;
 
-            // if startSample == endSample the cycle is empty!
-            if (startSample != endSample) {
-                moofsMdats.add(createMoof(startSample, endSample, track, sequence));
-                moofsMdats.add(createMdat(startSample, endSample, track, sequence++));
-            }
+        // if startSample == endSample the cycle is empty!
+        if (startSample != endSample) {
+            moofsMdats.add(createMoof(startSample, endSample, track, sequence));
+            moofsMdats.add(createMdat(startSample, endSample, track, sequence));
         }
         return sequence;
     }
@@ -154,15 +193,8 @@ public class FragmentedMp4Builder implements Mp4Builder {
      */
     public Container build(Movie movie) {
         LOG.fine("Creating movie " + movie);
-        if (intersectionFinder == null) {
-            Track refTrack = null;
-            for (Track track : movie.getTracks()) {
-                if (track.getHandler().equals("vide")) {
-                    refTrack = track;
-                    break;
-                }
-            }
-            intersectionFinder = new SyncSampleIntersectFinderImpl(movie, refTrack, -1);
+        if (fragmenter == null) {
+            fragmenter = new BetterFragmenter(2);
         }
         BasicContainer isoFile = new BasicContainer();
 
@@ -200,7 +232,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
             public long getSize() {
                 if (size_ != -1) return size_;
                 long size = 8; // I don't expect 2gig fragments
-                for (Sample sample : getSamples(startSample, endSample, track, i)) {
+                for (Sample sample : getSamples(startSample, endSample, track)) {
                     size += sample.getSize();
                 }
                 size_ = size;
@@ -218,7 +250,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
                 header.rewind();
                 writableByteChannel.write(header);
 
-                List<Sample> samples = getSamples(startSample, endSample, track, i);
+                List<Sample> samples = getSamples(startSample, endSample, track);
                 for (Sample sample : samples) {
                     sample.writeTo(writableByteChannel);
                 }
@@ -258,10 +290,10 @@ public class FragmentedMp4Builder implements Mp4Builder {
         createTfdt(startSample, track, traf);
         createTrun(startSample, endSample, track, sequenceNumber, traf);
 
-        if (track instanceof CencEncyprtedTrack) {
-            createSaiz(startSample, endSample, (CencEncyprtedTrack) track, sequenceNumber, traf);
-            createSenc(startSample, endSample, (CencEncyprtedTrack) track, sequenceNumber, traf);
-            createSaio(startSample, endSample, (CencEncyprtedTrack) track, sequenceNumber, traf);
+        if (track instanceof CencEncryptedTrack) {
+            createSaiz(startSample, endSample, (CencEncryptedTrack) track, sequenceNumber, traf);
+            createSenc(startSample, endSample, (CencEncryptedTrack) track, sequenceNumber, traf);
+            createSaio(startSample, endSample, (CencEncryptedTrack) track, sequenceNumber, traf);
         }
 
 
@@ -281,6 +313,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
             SampleGroupDescriptionBox sgpd = new SampleGroupDescriptionBox();
             String type = sg.getKey();
             sgpd.setGroupEntries(sg.getValue());
+            sgpd.setGroupingType(type);
             SampleToGroupBox sbgp = new SampleToGroupBox();
             sbgp.setGroupingType(type);
             SampleToGroupBox.Entry last = null;
@@ -290,7 +323,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
                     GroupEntry groupEntry = sg.getValue().get(j);
                     long[] sampleNums = track.getSampleGroups().get(groupEntry);
                     if (Arrays.binarySearch(sampleNums, i) >= 0) {
-                        index = j + 1;
+                        index = j + 0x10001;
                     }
                 }
                 if (last == null || last.getGroupDescriptionIndex() != index) {
@@ -305,17 +338,16 @@ public class FragmentedMp4Builder implements Mp4Builder {
         }
 
 
-
     }
 
-    protected void createSenc(long startSample, long endSample, CencEncyprtedTrack track, int sequenceNumber, TrackFragmentBox parent) {
+    protected void createSenc(long startSample, long endSample, CencEncryptedTrack track, int sequenceNumber, TrackFragmentBox parent) {
         SampleEncryptionBox senc = new SampleEncryptionBox();
         senc.setSubSampleEncryption(track.hasSubSampleEncryption());
         senc.setEntries(track.getSampleEncryptionEntries().subList(l2i(startSample - 1), l2i(endSample - 1)));
         parent.addBox(senc);
     }
 
-    protected void createSaio(long startSample, long endSample, CencEncyprtedTrack track, int sequenceNumber, TrackFragmentBox parent) {
+    protected void createSaio(long startSample, long endSample, CencEncryptedTrack track, int sequenceNumber, TrackFragmentBox parent) {
         SchemeTypeBox schm = Path.getPath(track.getSampleDescriptionBox(), "enc.[0]/sinf[0]/schm[0]");
 
         SampleAuxiliaryInformationOffsetsBox saio = new SampleAuxiliaryInformationOffsetsBox();
@@ -348,7 +380,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
 
     }
 
-    protected void createSaiz(long startSample, long endSample, CencEncyprtedTrack track, int sequenceNumber, TrackFragmentBox parent) {
+    protected void createSaiz(long startSample, long endSample, CencEncryptedTrack track, int sequenceNumber, TrackFragmentBox parent) {
         SampleDescriptionBox sampleDescriptionBox = track.getSampleDescriptionBox();
         SchemeTypeBox schm = Path.getPath(sampleDescriptionBox, "enc.[0]/sinf[0]/schm[0]");
         TrackEncryptionBox tenc = Path.getPath(sampleDescriptionBox, "enc.[0]/sinf[0]/schi[0]/tenc[0]");
@@ -376,13 +408,12 @@ public class FragmentedMp4Builder implements Mp4Builder {
      * Gets all samples starting with <code>startSample</code> (one based -&gt; one is the first) and
      * ending with <code>endSample</code> (exclusive).
      *
-     * @param startSample    low endpoint (inclusive) of the sample sequence
-     * @param endSample      high endpoint (exclusive) of the sample sequence
-     * @param track          source of the samples
-     * @param sequenceNumber the fragment index of the requested list of samples
-     * @return a <code>List&lt;ByteBuffer&gt;</code> of raw samples
+     * @param startSample low endpoint (inclusive) of the sample sequence
+     * @param endSample   high endpoint (exclusive) of the sample sequence
+     * @param track       source of the samples
+     * @return a <code>List&lt;Sample&gt;</code> of raw samples
      */
-    protected List<Sample> getSamples(long startSample, long endSample, Track track, int sequenceNumber) {
+    protected List<Sample> getSamples(long startSample, long endSample, Track track) {
         // since startSample and endSample are one-based substract 1 before addressing list elements
         return track.getSamples().subList(l2i(startSample) - 1, l2i(endSample) - 1);
     }
@@ -397,7 +428,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
      * @return the sample sizes in the given interval
      */
     protected long[] getSampleSizes(long startSample, long endSample, Track track, int sequenceNumber) {
-        List<Sample> samples = getSamples(startSample, endSample, track, sequenceNumber);
+        List<Sample> samples = getSamples(startSample, endSample, track);
 
         long[] sampleSizes = new long[samples.size()];
         for (int i = 0; i < sampleSizes.length; i++) {
@@ -828,7 +859,7 @@ public class FragmentedMp4Builder implements Mp4Builder {
 
             for (Edit edit : track.getEdits()) {
                 entries.add(new EditListBox.Entry(elst,
-                        (long) edit.getSegmentDuration() * movie.getTimescale(),
+                        Math.round(edit.getSegmentDuration() * movie.getTimescale()),
                         edit.getMediaTime() * track.getTrackMetaData().getTimescale() / edit.getTimeScale(),
                         edit.getMediaRate()));
             }
@@ -852,12 +883,12 @@ public class FragmentedMp4Builder implements Mp4Builder {
         return dinf;
     }
 
-    public FragmentIntersectionFinder getFragmentIntersectionFinder() {
-        return intersectionFinder;
+    public Fragmenter getFragmenter() {
+        return fragmenter;
     }
 
-    public void setIntersectionFinder(FragmentIntersectionFinder intersectionFinder) {
-        this.intersectionFinder = intersectionFinder;
+    public void setFragmenter(Fragmenter fragmenter) {
+        this.fragmenter = fragmenter;
     }
 
 
