@@ -43,14 +43,8 @@ public class AppendTrack extends AbstractTrack {
     private static Logger LOG = Logger.getLogger(AppendTrack.class);
     Track[] tracks;
     SampleDescriptionBox stsd;
-
-    public static String appendTracknames (Track... tracks) {
-        String name = "";
-        for (Track track : tracks) {
-            name += track.getName() + " + ";
-        }
-        return name.substring(0, name.length() -3);
-    }
+    List<Sample> lists;
+    long[] decodingTimes;
 
     public AppendTrack(Track... tracks) throws IOException {
         super(appendTracknames(tracks));
@@ -66,6 +60,33 @@ public class AppendTrack extends AbstractTrack {
 
             }
         }
+        lists = new ArrayList<Sample>();
+
+        for (Track track : tracks) {
+            //System.err.println("Track " + track + " is about to be appended");
+            lists.addAll(track.getSamples());
+        }
+
+        int numSamples = 0;
+        for (Track track : tracks) {
+            numSamples += track.getSampleDurations().length;
+        }
+        decodingTimes = new long[numSamples];
+        int index = 0;
+        // should use system arraycopy but this works too (yes it's slow ...)
+        for (Track track : tracks) {
+            long[] durs = track.getSampleDurations();
+            System.arraycopy(durs, 0, decodingTimes, index, durs.length);
+            index += durs.length;
+        }
+    }
+
+    public static String appendTracknames(Track... tracks) {
+        String name = "";
+        for (Track track : tracks) {
+            name += track.getName() + " + ";
+        }
+        return name.substring(0, name.length() - 3);
     }
 
     public void close() throws IOException {
@@ -192,9 +213,9 @@ public class AppendTrack extends AbstractTrack {
     private AudioSampleEntry mergeAudioSampleEntries(AudioSampleEntry ase1, AudioSampleEntry ase2) {
         AudioSampleEntry ase = new AudioSampleEntry(ase2.getType());
         if (ase1.getBytesPerFrame() == ase2.getBytesPerFrame()) {
-            LOG.logError("BytesPerFrame differ");
             ase.setBytesPerFrame(ase1.getBytesPerFrame());
         } else {
+            LOG.logError("BytesPerFrame differ");
             return null;
         }
         if (ase1.getBytesPerPacket() == ase2.getBytesPerPacket()) {
@@ -203,13 +224,12 @@ public class AppendTrack extends AbstractTrack {
             return null;
         }
         if (ase1.getBytesPerSample() == ase2.getBytesPerSample()) {
-            LOG.logError("BytesPerSample differ");
             ase.setBytesPerSample(ase1.getBytesPerSample());
         } else {
+            LOG.logError("BytesPerSample differ");
             return null;
         }
         if (ase1.getChannelCount() == ase2.getChannelCount()) {
-            LOG.logError("ChannelCount differ");
             ase.setChannelCount(ase1.getChannelCount());
         } else {
             return null;
@@ -217,6 +237,7 @@ public class AppendTrack extends AbstractTrack {
         if (ase1.getPacketSize() == ase2.getPacketSize()) {
             ase.setPacketSize(ase1.getPacketSize());
         } else {
+            LOG.logError("ChannelCount differ");
             return null;
         }
         if (ase1.getCompressionId() == ase2.getCompressionId()) {
@@ -366,11 +387,7 @@ public class AppendTrack extends AbstractTrack {
     }
 
     public List<Sample> getSamples() {
-        ArrayList<Sample> lists = new ArrayList<Sample>();
 
-        for (Track track : tracks) {
-            lists.addAll(track.getSamples());
-        }
 
         return lists;
     }
@@ -380,18 +397,6 @@ public class AppendTrack extends AbstractTrack {
     }
 
     public synchronized long[] getSampleDurations() {
-        int numSamples = 0;
-        for (Track track : tracks) {
-            numSamples += track.getSampleDurations().length;
-        }
-        long[] decodingTimes = new long[numSamples];
-        int index = 0;
-        // should use system arraycopy but this works too (yes it's slow ...)
-        for (Track track : tracks) {
-            for (long l : track.getSampleDurations()) {
-                decodingTimes[index++] = l;
-            }
-        }
         return decodingTimes;
     }
 
@@ -423,15 +428,17 @@ public class AppendTrack extends AbstractTrack {
         if (tracks[0].getSyncSamples() != null && tracks[0].getSyncSamples().length > 0) {
             int numSyncSamples = 0;
             for (Track track : tracks) {
-                numSyncSamples += track.getSyncSamples().length;
+                numSyncSamples += track.getSyncSamples() != null ? track.getSyncSamples().length : 0;
             }
             long[] returnSyncSamples = new long[numSyncSamples];
 
             int pos = 0;
             long samplesBefore = 0;
             for (Track track : tracks) {
-                for (long l : track.getSyncSamples()) {
-                    returnSyncSamples[pos++] = samplesBefore + l;
+                if (track.getSyncSamples() != null) {
+                    for (long l : track.getSyncSamples()) {
+                        returnSyncSamples[pos++] = samplesBefore + l;
+                    }
                 }
                 samplesBefore += track.getSamples().size();
             }
