@@ -5,18 +5,17 @@ import com.coremedia.iso.boxes.sampleentry.AudioSampleEntry;
 import com.googlecode.mp4parser.DataSource;
 import com.googlecode.mp4parser.authoring.AbstractTrack;
 import com.googlecode.mp4parser.authoring.Sample;
-import com.googlecode.mp4parser.authoring.SampleImpl;
 import com.googlecode.mp4parser.authoring.TrackMetaData;
 import com.googlecode.mp4parser.boxes.DTSSpecificBox;
 import com.googlecode.mp4parser.boxes.mp4.objectdescriptors.BitReaderBuffer;
 
+import java.io.EOFException;
 import java.io.IOException;
+import java.nio.Buffer;
 import java.nio.ByteBuffer;
 import java.nio.channels.WritableByteChannel;
 import java.util.*;
 
-import static com.googlecode.mp4parser.util.CastUtils.l2i;
-
 
 public class DTSTrackImpl extends AbstractTrack {
     TrackMetaData trackMetaData = new TrackMetaData();
@@ -102,7 +101,7 @@ public class DTSTrackImpl extends AbstractTrack {
         trackMetaData.setLanguage(lang);
         trackMetaData.setTimescale(samplerate); // Audio tracks always use samplerate as timescale
 
-        samples = readSamples();
+
     }
 
 
@@ -139,7 +138,7 @@ public class DTSTrackImpl extends AbstractTrack {
         return "soun";
     }
 
-    private boolean parseDtshdhdr(int size, ByteBuffer bb) {
+    private void parseDtshdhdr(int size, ByteBuffer bb) {
         int hdrVersion = bb.getInt();
         int timeCodeHigh = bb.get();
         int timeCode = bb.getInt();
@@ -165,7 +164,6 @@ public class DTSTrackImpl extends AbstractTrack {
             bb.get();
             i++;
         }
-        return true;
     }
 
     private boolean parseCoressmd(int size, ByteBuffer bb) {
@@ -184,8 +182,8 @@ public class DTSTrackImpl extends AbstractTrack {
     }
 
     private boolean parseAuprhdr(int size, ByteBuffer bb) {
-        int audioPresIndex = bb.get();
-        int bitwAupresData = bb.getShort();
+        int audioPresIndex = bb.get();  // Audio_Pres_Index
+        int bitwAupresData = bb.getShort(); // Bitw_Aupres_Metadata
         int a = bb.get();
         int b = bb.getShort();
         maxSampleRate = (a << 16) | (b & 0xffff);
@@ -220,6 +218,9 @@ public class DTSTrackImpl extends AbstractTrack {
         return true;
     }
 
+    /**
+     * EXTSS_MD
+     */
     private boolean parseExtssmd(int size, ByteBuffer bb) {
         int a = bb.get();
         int b = bb.getShort();
@@ -246,25 +247,23 @@ public class DTSTrackImpl extends AbstractTrack {
         ByteBuffer bb = dataSource.map(0, 25000);
         int testHeader1 = bb.getInt();
         int testHeader2 = bb.getInt();
-        if (testHeader1 != 0x44545348 || (testHeader2 != 0x44484452)) { // This is NOT a DTSHD file that has some extra metadata
-            return false;
+        if (testHeader1 != 0x44545348 || (testHeader2 != 0x44484452)) { // ASCII: DTSHDHDR
+            throw new IOException("data does not start with 'DTSHDHDR' as required for a DTS-HD file");
         }
 
-        while ((testHeader1 != 0x5354524d || testHeader2 != 0x44415441) && bb.remaining() > 100) {
+        while ((testHeader1 != 0x5354524d || testHeader2 != 0x44415441) && bb.remaining() > 100) { // ASCII: STRMDATA
             int size = (int) bb.getLong();
-            if (testHeader1 == 0x44545348 && testHeader2 == 0x44484452) {
-                if (!parseDtshdhdr(size, bb)) {
-                    return false;
-                }
-            } else if (testHeader1 == 0x434f5245 && testHeader2 == 0x53534d44) {
+            if (testHeader1 == 0x44545348 && testHeader2 == 0x44484452) { // ASCII: DTSHDHDR
+                parseDtshdhdr(size, bb);
+            } else if (testHeader1 == 0x434f5245 && testHeader2 == 0x53534d44) { // ASCII: CORESSMD
                 if (!parseCoressmd(size, bb)) {
                     return false;
                 }
-            } else if (testHeader1 == 0x41555052 && testHeader2 == 0x2d484452) {
+            } else if (testHeader1 == 0x41555052 && testHeader2 == 0x2d484452) { // ASCII: AUPR-HDR
                 if (!parseAuprhdr(size, bb)) {
                     return false;
                 }
-            } else if (testHeader1 == 0x45585453 && testHeader2 == 0x535f4d44) {
+            } else if (testHeader1 == 0x45585453 && testHeader2 == 0x535f4d44) { // ASCII: EXTSS_MD
                 if (!parseExtssmd(size, bb)) {
                     return false;
                 }
@@ -276,7 +275,7 @@ public class DTSTrackImpl extends AbstractTrack {
             testHeader1 = bb.getInt();
             testHeader2 = bb.getInt();
         }
-        bb.getLong(); // Data size, not needed here
+        long dataSize = bb.getLong();
         dataOffset = bb.position();
 
         int amode = -1;
@@ -295,10 +294,11 @@ public class DTSTrackImpl extends AbstractTrack {
 
         boolean done = false;
 
+
         while (!done) {
             int offset = bb.position();
             int sync = bb.getInt();
-            if (sync == 0x7ffe8001) {
+            if (sync == 0x7ffe8001) { // DTS_SYNCWORD_CORE
                 if (corePresent == 1) {
                     done = true;
                 } else {
@@ -320,163 +320,12 @@ public class DTSTrackImpl extends AbstractTrack {
                     frameSize += fsize + 1;
                     amode = brb.readBits(6); // Calc channel layout from this
                     int sfreq = brb.readBits(4);
-                    switch (sfreq)
 
-                    {
-                        case 1:
-                            samplerate = 8000;
-                            break;
-
-                        case 2:
-                            samplerate = 16000;
-                            break;
-
-                        case 3:
-                            samplerate = 32000;
-                            break;
-
-                        case 6:
-                            samplerate = 11025;
-                            break;
-
-                        case 7:
-                            samplerate = 22050;
-                            break;
-
-                        case 8:
-                            samplerate = 44100;
-                            break;
-
-                        case 11:
-                            samplerate = 12000;
-                            break;
-
-                        case 12:
-                            samplerate = 24000;
-                            break;
-
-                        case 13:
-                            samplerate = 48000;
-                            break;
-
-                        default: // No other base samplerates allowed
-                            return false;
-
-                    }
+                    samplerate = getSampleRate(sfreq);
 
                     int rate = brb.readBits(5);
 
-                    switch (rate)
-
-                    {
-                        case 0:
-                            bitrate = 32;
-                            break;
-
-                        case 1:
-                            bitrate = 56;
-                            break;
-
-                        case 2:
-                            bitrate = 64;
-                            break;
-
-                        case 3:
-                            bitrate = 96;
-                            break;
-
-                        case 4:
-                            bitrate = 112;
-                            break;
-
-                        case 5:
-                            bitrate = 128;
-                            break;
-
-                        case 6:
-                            bitrate = 192;
-                            break;
-
-                        case 7:
-                            bitrate = 224;
-                            break;
-
-                        case 8:
-                            bitrate = 256;
-                            break;
-
-                        case 9:
-                            bitrate = 320;
-                            break;
-
-                        case 10:
-                            bitrate = 384;
-                            break;
-
-                        case 11:
-                            bitrate = 448;
-                            break;
-
-                        case 12:
-                            bitrate = 512;
-                            break;
-
-                        case 13:
-                            bitrate = 576;
-                            break;
-
-                        case 14:
-                            bitrate = 640;
-                            break;
-
-                        case 15:
-                            bitrate = 768;
-                            break;
-
-                        case 16:
-                            bitrate = 960;
-                            break;
-
-                        case 17:
-                            bitrate = 1024;
-                            break;
-
-                        case 18:
-                            bitrate = 1152;
-                            break;
-
-                        case 19:
-                            bitrate = 1280;
-                            break;
-
-                        case 20:
-                            bitrate = 1344;
-                            break;
-
-                        case 21:
-                            bitrate = 1408;
-                            break;
-
-                        case 22:
-                            bitrate = 1411;
-                            break;
-
-                        case 23:
-                            bitrate = 1472;
-                            break;
-
-                        case 24:
-                            bitrate = 1536;
-                            break;
-
-                        case 25:
-                            bitrate = -1; // Open, some other bitrate....
-                            break;
-
-                        default:
-                            return false;
-
-                    }
+                    bitrate = getBitRate(rate);
 
                     int fixedBit = brb.readBits(1);
                     if (fixedBit != 0)
@@ -550,7 +399,7 @@ public class DTSTrackImpl extends AbstractTrack {
                     }
                     bb.position(offset + fsize + 1);
                 }
-            } else if (sync == 0x64582025) {
+            } else if (sync == 0x64582025) { // DTS_SYNCWORD_SUBSTREAM
                 if (corePresent == -1) {
                     corePresent = 0;
                     samplesPerFrame = samplesPerFrameAtMaxFs;
@@ -611,8 +460,9 @@ public class DTSTrackImpl extends AbstractTrack {
                 }
                 bb.position(offset + nuExtSSFsize);
             } else {
-                done = true;
+                throw new IOException("No DTS_SYNCWORD_* found at " + bb.position());
             }
+
         }
         int fd = -1;
         switch (samplesPerFrame)
@@ -790,39 +640,325 @@ public class DTSTrackImpl extends AbstractTrack {
                 }
             }
         }
+        samples = generateSamples(dataSource, dataOffset, dataSize, corePresent);
+        sampleDurations = new long[samples.size()];
+        Arrays.fill(sampleDurations, samplesPerFrame );
+
         return true;
     }
 
-    private List<Sample> readSamples() throws IOException {
-        ArrayList<Sample> mySamples = new  ArrayList<Sample>(l2i(dataSource.size() / frameSize));
-        int position = dataOffset;
-        while (position + frameSize < dataSource.size()) {
+    private List<Sample> generateSamples(DataSource dataSource, int dataOffset, long dataSize, int corePresent) throws IOException {
+        LookAhead la = new LookAhead(dataSource, dataOffset, dataSize , corePresent);
+        ByteBuffer sample;
+        List<Sample> mySamples = new ArrayList<Sample>();
 
-            final int currentPosition = position;
+        while ((sample = la.findNextStart()) != null) {
+            final ByteBuffer finalSample = sample;
             mySamples.add(new Sample() {
                 public void writeTo(WritableByteChannel channel) throws IOException {
-                    dataSource.transferTo(currentPosition, frameSize, channel);
+                    channel.write((ByteBuffer) finalSample.rewind());
                 }
 
                 public long getSize() {
-                    return frameSize;
+                    return finalSample.rewind().remaining();
                 }
 
                 public ByteBuffer asByteBuffer() {
-                    try {
-                        return dataSource.map(currentPosition, frameSize);
-                    } catch (IOException e) {
-                        throw new RuntimeException(e);
-                    }
+                    return finalSample;
                 }
             });
-            position += frameSize;
-
+            //System.err.println(finalSample.remaining());
         }
-        sampleDurations = new long[mySamples.size()];
-        Arrays.fill(sampleDurations, samplesPerFrame * samplerate / trackMetaData.getTimescale());
+        System.err.println("all samples found");
         return mySamples;
     }
 
+    private int getBitRate(int rate) throws IOException {
+        int bitrate;
+        switch (rate)
 
-}
+        {
+            case 0:
+                bitrate = 32;
+                break;
+
+            case 1:
+                bitrate = 56;
+                break;
+
+            case 2:
+                bitrate = 64;
+                break;
+
+            case 3:
+                bitrate = 96;
+                break;
+
+            case 4:
+                bitrate = 112;
+                break;
+
+            case 5:
+                bitrate = 128;
+                break;
+
+            case 6:
+                bitrate = 192;
+                break;
+
+            case 7:
+                bitrate = 224;
+                break;
+
+            case 8:
+                bitrate = 256;
+                break;
+
+            case 9:
+                bitrate = 320;
+                break;
+
+            case 10:
+                bitrate = 384;
+                break;
+
+            case 11:
+                bitrate = 448;
+                break;
+
+            case 12:
+                bitrate = 512;
+                break;
+
+            case 13:
+                bitrate = 576;
+                break;
+
+            case 14:
+                bitrate = 640;
+                break;
+
+            case 15:
+                bitrate = 768;
+                break;
+
+            case 16:
+                bitrate = 960;
+                break;
+
+            case 17:
+                bitrate = 1024;
+                break;
+
+            case 18:
+                bitrate = 1152;
+                break;
+
+            case 19:
+                bitrate = 1280;
+                break;
+
+            case 20:
+                bitrate = 1344;
+                break;
+
+            case 21:
+                bitrate = 1408;
+                break;
+
+            case 22:
+                bitrate = 1411;
+                break;
+
+            case 23:
+                bitrate = 1472;
+                break;
+
+            case 24:
+                bitrate = 1536;
+                break;
+
+            case 25:
+                bitrate = -1; // Open, some other bitrate....
+                break;
+
+            default:
+                throw new IOException("Unknown bitrate value");
+
+        }
+        return bitrate;
+    }
+
+    private int getSampleRate(int sfreq) throws IOException {
+        int samplerate;
+        switch (sfreq)
+
+        {
+            case 1:
+                samplerate = 8000;
+                break;
+
+            case 2:
+                samplerate = 16000;
+                break;
+
+            case 3:
+                samplerate = 32000;
+                break;
+
+            case 6:
+                samplerate = 11025;
+                break;
+
+            case 7:
+                samplerate = 22050;
+                break;
+
+            case 8:
+                samplerate = 44100;
+                break;
+
+            case 11:
+                samplerate = 12000;
+                break;
+
+            case 12:
+                samplerate = 24000;
+                break;
+
+            case 13:
+                samplerate = 48000;
+                break;
+
+            default: // No other base samplerates allowed
+                throw new IOException("Unknown Sample Rate");
+
+        }
+        return samplerate;
+    }
+
+
+
+    private static final int BUFFER = 1024 * 1024 * 64;
+
+    class LookAhead {
+        private final int corePresent;
+        long bufferStartPos;
+        int inBufferPos = 0;
+        DataSource dataSource;
+        long dataEnd;
+        ByteBuffer buffer;
+
+        long start;
+
+        LookAhead(DataSource dataSource, long bufferStartPos, long dataSize, int corePresent) throws IOException {
+            this.dataSource = dataSource;
+            this.bufferStartPos = bufferStartPos;
+            this.dataEnd = dataSize + bufferStartPos;
+            this.corePresent = corePresent;
+            fillBuffer();
+        }
+
+        public ByteBuffer findNextStart() throws IOException {
+            try {
+                // If core DTS stream is present then sync word is 0x7FFE8001
+                // otherwise 0x64582025
+                while (corePresent==1?!this.nextFourEquals0x7FFE8001():!nextFourEquals0x64582025()) {
+                    this.discardByte();
+                }
+                this.discardNext4AndMarkStart();
+
+                while (corePresent==1?!this.nextFourEquals0x7FFE8001orEof():!nextFourEquals0x64582025orEof()) {
+                    this.discardQWord();
+                }
+                return this.getSample();
+            } catch (EOFException e) {
+                return null;
+            }
+        }
+
+
+        private void fillBuffer() throws IOException {
+            System.err.println("Fill Buffer");
+            buffer = dataSource.map(bufferStartPos, Math.min(dataEnd - bufferStartPos, BUFFER));
+        }
+
+        private boolean nextFourEquals0x64582025() throws IOException {
+            return nextFourEquals((byte) 100, (byte) 88, (byte) 32, (byte) 37);
+        }
+
+        private boolean nextFourEquals0x7FFE8001() throws IOException {
+            return nextFourEquals((byte) 127, (byte) -2, (byte) -128, (byte) 1);
+        }
+
+        private boolean nextFourEquals(byte a, byte b, byte c, byte d) throws IOException {
+            if (buffer.limit() - inBufferPos >= 4) {
+                return ((buffer.get(inBufferPos) ==  a &&
+                        buffer.get(inBufferPos + 1) == b &&
+                        buffer.get(inBufferPos + 2) == c &&
+                        (buffer.get(inBufferPos + 3) == d)));
+            }
+            if (bufferStartPos + inBufferPos + 4 >= dataSource.size()) {
+                throw new EOFException();
+            }
+            return false;
+        }
+        private boolean nextFourEquals0x64582025orEof() throws IOException {
+            return nextFourEqualsOrEof((byte) 100, (byte) 88, (byte) 32, (byte) 37);
+        }
+
+
+        private boolean nextFourEquals0x7FFE8001orEof() throws IOException {
+            return nextFourEqualsOrEof((byte) 127, (byte) -2, (byte) -128, (byte) 1);
+        }
+        private boolean nextFourEqualsOrEof(byte a, byte b, byte c, byte d) throws IOException {
+            if (buffer.limit() - inBufferPos >= 4) {
+                if (((bufferStartPos + inBufferPos) % (1024 * 1024)) == 0) {
+                    System.err.println("" + ((bufferStartPos + inBufferPos) / 1024 / 1024));
+                }
+                return ((buffer.get(inBufferPos) ==  a /*0x7F */&&
+                        buffer.get(inBufferPos + 1) == b/*0xfe*/ &&
+                        buffer.get(inBufferPos + 2) == c /*0x80*/ &&
+                        (buffer.get(inBufferPos + 3) == d)));
+            } else {
+                if (bufferStartPos + inBufferPos + 4 > dataEnd) {
+                    return bufferStartPos + inBufferPos == dataEnd;
+                } else {
+                    bufferStartPos = start;
+                    inBufferPos = 0;
+                    fillBuffer();
+                    return nextFourEquals0x7FFE8001();
+                }
+            }
+        }
+
+
+        private void discardByte() {
+            inBufferPos += 1;
+        }
+        private void discardQWord() {
+            inBufferPos += 4;
+        }
+
+
+        private void discardNext4AndMarkStart() {
+            start = bufferStartPos + inBufferPos;
+            inBufferPos += 4;
+        }
+
+        private ByteBuffer getSample() {
+            if (start >= bufferStartPos) {
+                buffer.position((int) (start - bufferStartPos));
+                Buffer sample = buffer.slice();
+                sample.limit((int) (inBufferPos - (start - bufferStartPos)));
+                return (ByteBuffer) sample;
+            } else {
+                throw new RuntimeException("damn! NAL exceeds buffer");
+                // this can only happen if NAL is bigger than the buffer
+            }
+
+        }
+    }
+
+
+}
\ No newline at end of file
