@@ -4,7 +4,8 @@ package com.googlecode.mp4parser.boxes.apple;
  * Created by sannies on 10/15/13.
  */
 public class AppleNameBox extends Utf8AppleDataBox {
-    public AppleNameBox() {
-        super("©nam");
+    public static final String TYPE = "©nam";
+	public AppleNameBox() {
+        super( TYPE );
     }
 }
