@@ -4,15 +4,19 @@ import java.nio.ByteBuffer;
 
 public class BitReaderBuffer {
 
-    private ByteBuffer buffer;
     int initialPos;
     int position;
+    private ByteBuffer buffer;
 
     public BitReaderBuffer(ByteBuffer buffer) {
         this.buffer = buffer;
         initialPos = buffer.position();
     }
 
+    public boolean readBool() {
+        return readBits(1) == 1;
+    }
+
     public int readBits(int i) {
         byte b = buffer.get(initialPos + position / 8);
         int v = b < 0 ? b + 256 : b;
