@@ -4,18 +4,22 @@ import java.nio.ByteBuffer;
 
 public class BitWriterBuffer {
 
-    private ByteBuffer buffer;
     int initialPos;
     int position = 0;
+    private ByteBuffer buffer;
 
     public BitWriterBuffer(ByteBuffer buffer) {
         this.buffer = buffer;
         this.initialPos = buffer.position();
     }
 
+    public void writeBool(boolean b) {
+        writeBits(b ? 1 : 0, 1);
+    }
+
     public void writeBits(int i, int numBits) {
-        assert i <= ((1 << numBits)-1): String.format("Trying to write a value bigger (%s) than the number bits (%s) allows. " +
-                "Please mask the value before writing it and make your code is really working as intended.", i, (1<<numBits)-1);
+        assert i <= ((1 << numBits) - 1) : String.format("Trying to write a value bigger (%s) than the number bits (%s) allows. " +
+                "Please mask the value before writing it and make your code is really working as intended.", i, (1 << numBits) - 1);
 
         int left = 8 - position % 8;
         if (numBits <= left) {
