@@ -23,6 +23,7 @@ package com.googlecode.mp4parser.h264.model;
 import com.googlecode.mp4parser.h264.read.CAVLCReader;
 import com.googlecode.mp4parser.h264.write.CAVLCWriter;
 
+import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -52,6 +53,7 @@ public class SeqParameterSet extends BitstreamElement {
     public int bit_depth_chroma_minus8;
     public boolean qpprime_y_zero_transform_bypass_flag;
     public int profile_idc;
+    public long reserved_zero_2bits;
     public boolean constraint_set_0_flag;
     public boolean constraint_set_1_flag;
     public boolean constraint_set_2_flag;
@@ -76,6 +78,11 @@ public class SeqParameterSet extends BitstreamElement {
     public ScalingMatrix scalingMatrix;
     public int num_ref_frames_in_pic_order_cnt_cycle;
 
+
+    public static SeqParameterSet read(byte[] b) throws IOException {
+        return read(new ByteArrayInputStream(b));
+    }
+
     public static SeqParameterSet read(InputStream is) throws IOException {
         CAVLCReader reader = new CAVLCReader(is);
         SeqParameterSet sps = new SeqParameterSet();
@@ -94,7 +101,7 @@ public class SeqParameterSet extends BitstreamElement {
         sps.constraint_set_5_flag = reader
                 .readBool("SPS: constraint_set_5_flag");
 
-        reader.readNBit(2, "SPS: reserved_zero_2bits");
+        sps.reserved_zero_2bits = reader.readNBit(2, "SPS: reserved_zero_2bits");
         sps.level_idc = (int) reader.readNBit(8, "SPS: level_idc");
         sps.seq_parameter_set_id = reader.readUE("SPS: seq_parameter_set_id");
 
